@@ -1,10 +1,19 @@
 import copy
 import json
-from pathlib import Path
-from threading import Thread
 import time
-from typing import Dict, List, Optional, Union
+from pathlib import Path
+from requests import HTTPError
+from threading import Thread
+from typing import Dict, List, Optional, Tuple, Union
 
+from anylearn.sdk import (
+    AlgorithmArtifact,
+    Artifact,
+    DatasetArtifact,
+    ModelArtifact,
+    Project as SDKProject,
+)
+from anylearn.utils.cmd import cmd_error, cmd_warning
 from anylearn.utils.func import generate_primary_key
 from anylearn.applications.train_profile import TrainProfile
 from .algorithm_manager import sync_algorithm
@@ -223,6 +232,7 @@ def _create_train_task(name: str,
                        hyperparams_prefix: str="--",
                        hyperparams_delimeter: str=" ",
                        mounts: Optional[Union[Dict[str, List[Resource]], Dict[str, Resource]]]=None,
+                       artifacts: Optional[List[Artifact]]=None,
                        algorithm_git_ref: Optional[str]=None,
                        resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None,
                        description: Optional[str]=None,			
@@ -232,21 +242,23 @@ def _create_train_task(name: str,
                        mirror_name: Optional[str]="QUICKSTART",
                        num_nodes: Optional[int]=1,
                        nproc_per_node: Optional[int]=1):
-    resource_ids = []
+    resource_ids = set()
     train_params = hyperparams
     for k, v in mounts.items():
         if isinstance(v, list):
-            resource_ids.extend([v_item.id for v_item in v])
+            resource_ids.update([v_item.id for v_item in v])
             train_params[k] = [f"\"${v_item.id}\"" for v_item in v]
         else:
-            resource_ids.append(v.id)
+            resource_ids.add(v.id)
             train_params[k] = f"\"${v.id}\""
+    for artifact in artifacts:
+        resource_ids.add(artifact.id)
     train_task = TrainTask(
         name=name,
         project_id=project.id,
         algorithm_id=algorithm.id,
         algorithm_git_ref=algorithm_git_ref,
-        files=resource_ids,
+        files=list(resource_ids),
         train_params=json.dumps(train_params),
         train_params_prefix=hyperparams_prefix,
         train_params_delimeter=hyperparams_delimeter,
@@ -287,29 +299,126 @@ def _create_train_task(name: str,
     return train_task
 
 
+def upload(
+    algorithm_cloud_name: str,
+    algorithm_entrypoint: str,
+    algorithm_output: Optional[str]=None,
+    algorithm_local_dir: Optional[Union[str, Path]]='.',
+    algorithm_force_update: bool=False,
+) -> Tuple[Algorithm, str]:
+    try:
+        return sync_algorithm(
+            name=algorithm_cloud_name,
+            dir_path=algorithm_local_dir,
+            force=algorithm_force_update,
+        )
+    except AnylearnRequiredLocalCommitException:
+        # Notify possible usage of algorithm_force_update=True
+        raise AnylearnRequiredLocalCommitException(
+            "Local algorithm code has uncommitted changes. "
+            "Please commit your changes or "
+            "specify `algorithm_force_update=True` "
+            "to let Anylearn make an auto-commit."
+    )
+
+
+def run(
+    algorithm_cloud_name: str,
+    algorithm_entrypoint: str,
+    algorithm_output: Optional[str]='./output/',
+    algorithm_hyperparams: dict={},
+    dataset_cloud_names: Optional[List[str]]=None,
+    model_cloud_names: Optional[List[str]]=None,
+    project_name: Optional[str]=None,
+    image_name: Optional[str]="QUICKSTART",
+    quota_group_request: Optional[Dict[str, int]]=None,
+) -> Optional[TrainTask]:
+    kw = locals()
+
+    # Fetch algorithm by full name
+    try:
+        algo = AlgorithmArtifact.from_full_name(algorithm_cloud_name)
+    except HTTPError as e:
+        cmd_error(f"Algorithm {algorithm_cloud_name} not found.")
+        return None
+    kw['algorithm_id'] = algo.id
+
+    # Fetch project by full name
+    if project_name:
+        try:
+            project = SDKProject.from_full_name(kw.pop('project_name'))
+        except HTTPError as e:
+            if e.response.status_code == 404:
+                cmd_error(f"Project {project_name} not found.")
+                return None
+        kw['project_id'] = project.id
+
+    t, _, _, _ = quick_train(**kw)
+    return t
+
+
+def upload_and_run(
+    algorithm_cloud_name: str,
+    algorithm_entrypoint: str,
+    algorithm_output: Optional[str]='./output/',
+    algorithm_hyperparams: dict={},
+    algorithm_local_dir: Optional[Union[str, Path]]='.',
+    algorithm_force_update: bool=False,
+    dataset_cloud_names: Optional[List[str]]=None,
+    model_cloud_names: Optional[List[str]]=None,
+    project_name: Optional[str]=None,
+    image_name: Optional[str]="QUICKSTART",
+    quota_group_request: Optional[Dict[str, int]]=None,
+) -> TrainTask:
+    kw = locals()
+
+    # Fetch project by full name
+    if project_name:
+        try:
+            project = SDKProject.from_full_name(kw.pop('project_name'))
+        except HTTPError as e:
+            if e.response.status_code == 404:
+                cmd_error(f"Project {project_name} not found.")
+                return None
+        kw['project_id'] = project.id
+
+    t, _, _, _ = quick_train(**kw)
+    return t
+
+
 def quick_train(algorithm_id: Optional[str]=None,
                 algorithm_name: Optional[str]=None,
+                algorithm_cloud_name: Optional[str]=None,
                 algorithm_dir: Optional[Union[str, Path]]=None,
+                algorithm_local_dir: Optional[Union[str, Path]]=None,
                 algorithm_force_update: bool=False,
                 algorithm_git_ref: Optional[str]=None,
                 dataset_hyperparam_name: str="dataset",
                 dataset_id: Optional[Union[List[str], str]]=None,
                 dataset_dir: Optional[Union[str, Path]]=None,
                 dataset_archive: Optional[str]=None,
+                dataset_cloud_names: Optional[List[str]]=None,
                 model_hyperparam_name: str="model",
                 model_id: Optional[Union[List[str], str]]=None,
+                model_cloud_names: Optional[List[str]]=None,
                 pretrain_hyperparam_name: str="pretrain",
                 pretrain_task_id: Optional[Union[List[str], str]]=None,
                 project_id: Optional[str]=None,
                 project_name: Optional[str]=None,
                 entrypoint: Optional[str]=None,
+                algorithm_entrypoint: Optional[str]=None,
                 output: Optional[str]=None,
+                algorithm_output: Optional[str]='./output/',
                 mirror_name: Optional[str]="QUICKSTART",
+                image_name: Optional[str]="QUICKSTART",
                 resource_uploader: Optional[ResourceUploader]=None,
                 resource_polling: Union[float, int]=5,
                 hyperparams: dict={},
                 hyperparams_prefix: str="--",
                 hyperparams_delimeter: str=" ",
+                algorithm_hyperparams: dict={},
+                algorithm_hyperparams_prefix: str="--",
+                algorithm_hyperparams_delimeter: str=" ",
                 resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None,
                 quota_group_name: Optional[str]=None,
                 quota_group_request: Optional[Dict[str, int]]=None,
@@ -355,8 +464,30 @@ def quick_train(algorithm_id: Optional[str]=None,
         如有重复，则复用已存在的同名算法，
         算法文件将被覆盖并提升版本。
         原有版本仍可追溯。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_cloud_name` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_cloud_name` instead.
+            remove in 0.18.0.
+    algorithm_cloud_name: :obj:`str`, optional
+        指定的算法在Anylearn云环境中的名称。
+        同一用户的自定义算法的名称不可重复。
+        如有重复，则复用已存在的同名算法，
+        算法文件将被覆盖并提升版本。
+        原有版本仍可追溯。
     algorithm_dir : :obj:`str`, optional
         本地算法目录路径。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_local_dir` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_local_dir` instead.
+            remove in 0.18.0.
+    algorithm_local_dir : :obj:`str`, optional
+        本地算法目录路径。
     algorithm_git_ref : :obj:`str`, optional
         算法Gitea代码仓库的版本号（可以是commit号、分支名、tag名）。
         使用本地算法时，如未提供此参数，则取本地算法当前分支名。
@@ -371,8 +502,24 @@ def quick_train(algorithm_id: Optional[str]=None,
         已在Anylearn远程注册的数据集ID。
     dataset_dir : :obj:`str`, optional
         本地数据集目录路径。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        数据集上传将不再被支持。
+        
+        .. deprecated:: 0.17.0
+            
+            Uploading datasets will no longer be supported.
+            remove in 0.18.0.
     dataset_archive : :obj:`str`, optional
         本地数据集压缩包路径。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        数据集上传将不再被支持。
+        
+        .. deprecated:: 0.17.0
+            
+            Uploading datasets will no longer be supported.
+            remove in 0.18.0.
+    dataset_cloud_names : :obj:`List[str]`, optional
+        训练任务需使用的Anylearn云环境中的数据集的名称
     model_hyperparam_name : :obj:`str`, optional
         启动训练时，模型路径作为启动命令参数传入算法的参数名。
         需指定长参数名，如 :obj:`--model` ，并省略 :obj:`--` 部分传入。
@@ -380,6 +527,8 @@ def quick_train(algorithm_id: Optional[str]=None,
         默认为 :obj:`model` 。
     model_id : :obj:`str`, optional
         已在Anylearn远程注册/转存的模型ID。
+    model_cloud_names : :obj:`List[str]`, optional
+        训练任务需使用的Anylearn云环境中的模型的名称
     pretrain_hyperparam_name: :obj:`str`, optional
         启动训练时，前置训练结果（间接抽象为“预训练”，即"pretrain"）路径作为启动命令参数传入算法的参数名。
         需指定长参数名，如 :obj:`--pretrain` ，并省略 :obj:`--` 部分传入。
@@ -392,37 +541,119 @@ def quick_train(algorithm_id: Optional[str]=None,
         已在Anylearn远程创建的训练项目ID。
     entrypoint : :obj:`str`, optional
         启动训练的入口命令。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_entrypoint` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_entrypoint` instead.
+            remove in 0.18.0.
+    algorithm_entrypoint : :obj:`str`, optional
+        启动训练的入口命令。
     output : :obj:`str`, optional
         训练输出模型的相对路径（相对于算法目录）。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_output` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_output` instead.
+            remove in 0.18.0.
+    algorithm_output : :obj:`str`, optional
+        训练输出模型的相对路径（相对于算法目录）。
+    mirror_name : :obj:`str`, optional
+        训练使用的Anylearn云环境中的镜像的名称。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`image_name` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`image_name` instead.
+            remove in 0.18.0.
+    image_name : :obj:`str`, optional
+        训练使用的Anylearn云环境中的镜像的名称。
     resource_uploader : :obj:`ResourceUploader`, optional
         资源上传实现。
         默认使用系统内置的同步上传器 :obj:`SyncResourceUploader` 。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        上传器将不再需要。
+        
+        .. deprecated:: 0.17.0
+            
+            Uploaders will no longer be needed.
+            remove in 0.18.0.
     resource_polling : :obj:`float|int`, optional
         资源上传中轮询资源状态的时间间隔（单位：秒）。
         默认为5秒。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        轮询将不再需要。
+        
+        .. deprecated:: 0.17.0
+            
+            Polling will no longer be needed.
+            remove in 0.18.0.
     hyperparams : :obj:`dict`, optional
         训练超参数字典。
         超参数将作为训练启动命令的参数传入算法。
         超参数字典中的键应为长参数名，如 :obj:`--param` ，并省略 :obj:`--` 部分传入。
         如需要标识类参数（flag），可将参数的值设为空字符串，如 :obj:`{'my-flag': ''}` ，等价于 :obj:`--my-flag` 传入训练命令。
         默认为空字典。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_hyperparams` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_hyperparams` instead.
+            remove in 0.18.0.
     hyperparams_prefix : :obj:`str`, optional
         训练超参数键前标识，可支持hydra特殊命令行传参格式的诸如 :obj:`+key1` 、 :obj:`++key2` 、 空前置 :obj:`key3` 等需求，
         默认为 :obj:`--` 。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_hyperparams_prefix` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_hyperparams_prefix` instead.
+            remove in 0.18.0.
     hyperparams_delimeter :obj:`str`, optional
         训练超参数键值间的分隔符，默认为空格 :obj:` ` 。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`algorithm_hyperparams_delimeter` 作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            use :obj:`algorithm_hyperparams_delimeter` instead.
+            remove in 0.18.0.
+    algorithm_hyperparams : :obj:`dict`, optional
+        训练超参数字典。
+        超参数将作为训练启动命令的参数传入算法。
+        超参数字典中的键应为长参数名，如 :obj:`--param` ，并省略 :obj:`--` 部分传入。
+        如需要标识类参数（flag），可将参数的值设为空字符串，如 :obj:`{'my-flag': ''}` ，等价于 :obj:`--my-flag` 传入训练命令。
+        默认为空字典。
+    algorithm_hyperparams_prefix : :obj:`str`, optional
+        训练超参数键前标识，可支持hydra特殊命令行传参格式的诸如 :obj:`+key1` 、 :obj:`++key2` 、 空前置 :obj:`key3` 等需求，
+        默认为 :obj:`--` 。
+    algorithm_hyperparams_delimeter :obj:`str`, optional
+        训练超参数键值间的分隔符，默认为空格 :obj:` ` 。
     resource_request : :obj:`List[Dict[str, Dict[str, int]]]`, optional
         训练所需计算资源的请求。
         如未填，则使用Anylearn后端的 :obj:`default` 资源组中的默认资源套餐。
-        自0.13.1版本起，此参数被标记为废弃，将于0.14.0版本中移除。
-        请使用 :obj:`quota_group_name` 和 :obj:`quota_group_request` 作为替代。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请使用 :obj:`quota_group_request` 作为替代。
         
-        .. deprecated:: 0.13.1
+        .. deprecated:: 0.17.0
             
-            use :obj:`quota_group_name` and :obj:`quota_group_request` instead.
-            remove in 0.14.0.
+            use :obj:`quota_group_request` instead.
+            remove in 0.18.0.
     quota_group_name : :obj:`str`, optional
         训练所需计算资源组名称或ID。
+        自0.17.0版本起，此参数被标记为废弃，将于0.18.0版本中移除。
+        请在 :obj:`quota_group_request['name']` 字段中指明资源组名称作为替代。
+        
+        .. deprecated:: 0.17.0
+            
+            Please specify the quota group name in :obj:`quota_group_request['name']` instead."
+            remove in 0.18.0.
     quota_group_request : :obj:`dict`, optional
         训练所需计算资源组中资源数量。
         若 :obj:`quota_group_name` 和 :obj:`quota_group_request` 有其一未填，则使用Anylearn后端的 :obj:`default` 资源组中的默认资源套餐。
@@ -448,12 +679,78 @@ def quick_train(algorithm_id: Optional[str]=None,
     Project
         创建的训练项目对象
     """
+    # Deprecations
+    if algorithm_name is not None:
+        cmd_warning(
+            "Argument `algorithm_name` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please use `algorithm_cloud_name` instead."
+        )
+    if algorithm_dir is not None:
+        cmd_warning(
+            "Argument `algorithm_dir` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please use `algorithm_local_dir` instead."
+        )
+    if dataset_dir is not None:
+        cmd_warning(
+            "Argument `dataset_dir` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Uploading datasets will no longer be supported in `quick_train`"
+        )
+    if dataset_archive is not None:
+        cmd_warning(
+            "Argument `dataset_archive` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Uploading datasets will no longer be supported in `quick_train`"
+        )
+    if entrypoint is not None:
+        cmd_warning(
+            "Argument `entrypoint` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please use `algorithm_entrypoint` instead."
+        )
+    if output is not None:
+        cmd_warning(
+            "Argument `output` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please use `algorithm_output` instead."
+        )
+    if resource_uploader is not None:
+        cmd_warning(
+            "Argument `resource_uploader` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Uploaders will no longer be needed in `quick_train`"
+        )
+    if resource_polling != 5:
+        cmd_warning(
+            "Argument `resource_polling` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Polling will no longer be needed in `quick_train`"
+        )
+    if resource_request is not None:
+        cmd_warning(
+            "Argument `resource_request` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please use `quota_group_request` instead."
+        )
+    if quota_group_name is not None:
+        cmd_warning(
+            "Argument `quota_group_name` is deprecated "
+            "and will be removed in v0.18.0. "
+            "Please specify the quota group name in "
+            "`quota_group_request['name']` instead."
+        )
+
     # Resource request
     if quota_group_name and quota_group_request:
         resource_request = [{quota_group_name: quota_group_request}]
+    elif quota_group_request and 'name' in quota_group_request:
+        quota_group_name = quota_group_request.pop('name')
+        resource_request = [{quota_group_name: quota_group_request}]
     resource_request = _format_resource_request(resource_request)
 
-    # Remote odel
+    # Remote model
     if model_id is None:
         model_id = []
     elif not isinstance(model_id, list):
@@ -483,13 +780,30 @@ def quick_train(algorithm_id: Optional[str]=None,
         for _id in pretrain_task_id 
     ]
 
+    artifacts: List[Artifact] = []
+    # Remote dataset by full names
+    if dataset_cloud_names:
+        artifacts.extend([
+            DatasetArtifact.from_full_name(_fn)
+            for _fn in dataset_cloud_names
+        ])
+    # Remote model by full names
+    if model_cloud_names:
+        artifacts.extend([
+            ModelArtifact.from_full_name(_fn)
+            for _fn in model_cloud_names['models']
+        ])
+
     # Algorithm
+    _algo_name = algorithm_name or algorithm_cloud_name
+    _algo_dir = algorithm_dir or algorithm_local_dir
+    _image_name = mirror_name or image_name
     try:
         algo, current_sha = sync_algorithm(
             id=algorithm_id,
-            name=algorithm_name,
-            dir_path=algorithm_dir,
-            mirror_name=mirror_name,
+            name=_algo_name,
+            dir_path=_algo_dir,
+            mirror_name=_image_name,
             uploader=resource_uploader,
             polling=resource_polling,
             force=algorithm_force_update,
@@ -540,23 +854,29 @@ def quick_train(algorithm_id: Optional[str]=None,
 
     # Train task
     train_task_name = generate_random_name()
+    _hyperparams = hyperparams or algorithm_hyperparams
+    _hyperparams_prefix = hyperparams_prefix or algorithm_hyperparams_prefix
+    _hyperparams_delimeter = hyperparams_delimeter or algorithm_hyperparams_delimeter
+    _entrypoint = entrypoint or algorithm_entrypoint
+    _output = output or algorithm_output
     train_task = _create_train_task(
         name=train_task_name,
         algorithm=algo,
         algorithm_git_ref=algorithm_git_ref or current_sha,
         project=project,
-        hyperparams=hyperparams,
-        hyperparams_prefix=hyperparams_prefix,
-        hyperparams_delimeter=hyperparams_delimeter,
+        hyperparams=_hyperparams,
+        hyperparams_prefix=_hyperparams_prefix,
+        hyperparams_delimeter=_hyperparams_delimeter,
         mounts=mounts,
+        artifacts=artifacts,
         resource_request=resource_request,
         description=task_description,
         num_nodes=num_nodes,
         nproc_per_node=nproc_per_node,
-        entrypoint=entrypoint,
-        output=output,
-        algorithm_dir=algorithm_dir,
-        mirror_name=mirror_name,
+        entrypoint=_entrypoint,
+        output=_output,
+        algorithm_dir=_algo_dir,
+        mirror_name=_image_name,
     )
 
     DB().create_or_update_train_task(train_task=train_task)
